hyperlooprailpictures = function()
  return hyperlooprailpicturesinternal({{"metals", "metals"}, {"backplates", "backplates"}, {"ties", "ties"}, {"stone_path", "stone-path"}})
end

hyperlooprailpicturesinternal = function(elems)
  local keys = {{"hyperloop-straight_rail", "horizontal", 64, 64},
                {"hyperloop-straight_rail", "vertical", 64, 64},
                {"hyperloop-straight_rail", "diagonal", 64, 64},
                {"hyperloop-curved_rail", "vertical", 128, 256},
                {"hyperloop-curved_rail" ,"horizontal", 256, 128}}
  local res = {}
  for _ , key in ipairs(keys) do
    part = {}
    dashkey = key[1]:gsub("_", "-")
    for _ , elem in ipairs(elems) do
      part[elem[1]] = {
        filename = string.format("__Hyperloop__/graphics/%s/%s-%s-%s.png", dashkey, dashkey, key[2], elem[2]),
        priority = "extra-high",
        width = key[3],
        height = key[4]
      }
    end
    res[key[1] .. "_" .. key[2]] = part
  end
  res["rail_endings"] = {
    sheet =
    {
      filename = "__Hyperloop__/graphics/hyperloop-rail-endings/rail-endings.png",
      priority = "high",
      width = 88,
      height = 82
    }
  }
  return res
end

data:extend(
{
	{
		type = "straight-rail",
		name = "hyperloop-straight-rail",
		icon = "__Hyperloop__/graphics/icons/hyperloop-straight-rail.png",
		flags = {"placeable-neutral", "player-creation", "building-direction-8-way"},
		minable = {mining_time = 1, result = "hyperloop-straight-rail"},
		max_health = 100,
		corpse = "straight-rail-remnants",
		collision_box = {{-0.7, -0.8}, {0.7, 0.8}},
		selection_box = {{-0.7, -0.8}, {0.7, 0.8}},
		rail_category = "hyperloop-rail",
		pictures = railpictures(),
	},
	{
		type = "curved-rail",
		name = "hyperloop-curved-rail",
		icon = "__Hyperloop__/graphics/icons/hyperloop-curved-rail.png",
		flags = {"placeable-neutral", "player-creation", "building-direction-8-way"},
		minable = {mining_time = 1, result = "hyperloop-curved-rail"},
		max_health = 200,
		corpse = "curved-rail-remnants",
		collision_box = {{-0.75, -0.55}, {0.75, 1.6}},
		secondary_collision_box = {{-0.65, -2.43}, {0.65, 2.43}},
		selection_box = {{-1.7, -0.8}, {1.7, 0.8}},
		rail_category = "hyperloop-rail",
		pictures = railpictures(),
	},
  	{
		type = "locomotive",
		name = "hyperloop-locomotive",
		icon = "__Hyperloop__/graphics/icons/hyperloop-locomotive.png",
		flags = {"placeable-neutral", "player-creation", "placeable-off-grid", "not-on-map"},
		minable = {mining_time = 1, result = "hyperloop-locomotive"},
		mined_sound = {filename = "__core__/sound/deconstruct-medium.ogg"},
		max_health = 1000,
		corpse = "medium-remnants",
		dying_explosion = "medium-explosion",
		collision_box = {{-0.6, -2.6}, {0.6, 2.6}},
		selection_box = {{-0.85, -2.6}, {0.9, 2.5}},
		drawing_box = {{-1, -4}, {1, 3}},
		weight = 2000,
		max_speed = 10,
		max_power = "5000kW",
		braking_force = 25,
		friction_force = 0.000001,
		-- this is a percentage of current speed that will be subtracted
		air_resistance = 0.000001,
		connection_distance = 3.3,
		joint_distance = 4.6,
		energy_per_hit_point = 5,
		resistances =
		{
		  {
			type = "fire",
			decrease = 15,
			percent = 50
		  },
		  {
			type = "physical",
			decrease = 15,
			percent = 30
		  },
		  {
			type = "impact",
			decrease = 50,
			percent = 60
		  },
		  {
			type = "explosion",
			decrease = 15,
			percent = 30
		  },
		  {
			type = "acid",
			decrease = 10,
			percent = 20
		  }
		},
		energy_source =
		{
			type = "electric",
			usage_priority = "secondary-input",
			drain = "20000kW"
		},
		--[[energy_source =
		{
			type = "burner",
			effectivity = 10,
			fuel_inventory_size = 5,
			smoke =
			{
				{
					name = "train-smoke",
					deviation = {0.3, 0.3},
					frequency = 100,
					position = {0, 0},
					starting_frame = 0,
					starting_frame_deviation = 60,
					height = 2,
					height_deviation = 0.5,
					starting_vertical_speed = 0.2,
					starting_vertical_speed_deviation = 0.1,
				}
			}
		},--]]
		front_light =
		{
		  {
			type = "oriented",
			minimum_darkness = 0.3,
			picture =
			{
			  filename = "__core__/graphics/light-cone.png",
			  priority = "medium",
			  scale = 2,
			  width = 200,
			  height = 200
			},
			shift = {-0.6, -16},
			size = 2,
			intensity = 0.6
		  },
		  {
			type = "oriented",
			minimum_darkness = 0.3,
			picture =
			{
			  filename = "__core__/graphics/light-cone.png",
			  priority = "medium",
			  scale = 2,
			  width = 200,
			  height = 200
			},
			shift = {0.6, -16},
			size = 2,
			intensity = 0.6
		  }
		},
		back_light = rolling_stock_back_light(),
		stand_by_light = rolling_stock_stand_by_light(),
		pictures =
		{
		  priority = "very-low",
		  width = 346,
		  height = 248,
		  direction_count = 256,
		  filenames =
		  {
			"__Hyperloop__/graphics/hyperloop-locomotive/hyperloop-locomotive-01.png",
			"__Hyperloop__/graphics/hyperloop-locomotive/hyperloop-locomotive-02.png",
			"__Hyperloop__/graphics/hyperloop-locomotive/hyperloop-locomotive-03.png",
			"__Hyperloop__/graphics/hyperloop-locomotive/hyperloop-locomotive-04.png",
			"__Hyperloop__/graphics/hyperloop-locomotive/hyperloop-locomotive-05.png",
			"__Hyperloop__/graphics/hyperloop-locomotive/hyperloop-locomotive-06.png",
			"__Hyperloop__/graphics/hyperloop-locomotive/hyperloop-locomotive-07.png",
			"__Hyperloop__/graphics/hyperloop-locomotive/hyperloop-locomotive-08.png"
		  },
		  line_length = 4,
		  lines_per_file = 8,
		  shift = {1.0, -0.45}
		},
		rail_category = "hyperloop-rail",
		stop_trigger =
		{
		  -- left side
		  {
			type = "create-smoke",
			repeat_count = 125,
			entity_name = "smoke-train-stop",
			initial_height = 0,
			-- smoke goes to the left
			speed = {-0.03, 0},
			speed_multiplier = 0.75,
			speed_multiplier_deviation = 1.1,
			offset_deviation = {{-0.75, -2.7}, {-0.3, 2.7}}
		  },
		  -- right side
		  {
			type = "create-smoke",
			repeat_count = 125,
			entity_name = "smoke-train-stop",
			initial_height = 0,
			-- smoke goes to the right
			speed = {0.03, 0},
			speed_multiplier = 0.75,
			speed_multiplier_deviation = 1.1,
			offset_deviation = {{0.3, -2.7}, {0.75, 2.7}}
		  },
		  {
			type = "play-sound",
			sound =
			{
			  {
				filename = "__base__/sound/train-breaks.ogg",
				volume = 0.6
			  },
			}
		  },
		},
		drive_over_tie_trigger = drive_over_tie(),
		tie_distance = 50,
		vehicle_impact_sound =  { filename = "__base__/sound/car-metal-impact.ogg", volume = 0.65 },
		working_sound =
		{
		  sound =
		  {
			filename = "__base__/sound/train-engine.ogg",
			volume = 0.4
		  },
		  match_speed_to_activity = true,
		},
		open_sound = { filename = "__base__/sound/car-door-open.ogg", volume=0.7 },
		close_sound = { filename = "__base__/sound/car-door-close.ogg", volume = 0.7 },
		sound_minimum_speed = 0.5;
	},
	{
		type = "cargo-wagon",
		name = "hyperloop-cargo-wagon",
		icon = "__Hyperloop__/graphics/icons/hyperloop-cargo-wagon.png",
		flags = {"placeable-neutral", "player-creation", "placeable-off-grid", "not-on-map"},
		inventory_size = 30,
		minable = {mining_time = 1, result = "hyperloop-cargo-wagon"},
		mined_sound = {filename = "__core__/sound/deconstruct-medium.ogg"},
		max_health = 600,
		corpse = "medium-remnants",
		dying_explosion = "medium-explosion",
		collision_box = {{-0.6, -2.4}, {0.6, 2.4}},
		selection_box = {{-0.7, -2.5}, {1, 2.5}},
		weight = 1000,
		max_speed = 10,
		braking_force = 10,
		friction_force = 0.000001,
		air_resistance = 0.000001,
		connection_distance = 3.3,
		joint_distance = 4,
		energy_per_hit_point = 5,
		resistances =
		{
		  {
			type = "fire",
			decrease = 15,
			percent = 50
		  },
		  {
			type = "physical",
			decrease = 15,
			percent = 30
		  },
		  {
			type = "impact",
			decrease = 50,
			percent = 60
		  },
		  {
			type = "explosion",
			decrease = 15,
			percent = 30
		  },
		  {
			type = "acid",
			decrease = 10,
			percent = 20
		  }
		},
		back_light = rolling_stock_back_light(),
		stand_by_light = rolling_stock_stand_by_light(),
		pictures =
		{
		  priority = "very-low",
		  width = 285,
		  height = 218,
		  back_equals_front = true,
		  direction_count = 128,
		  filenames =
		  {
			"__Hyperloop__/graphics/hyperloop-cargo-wagon/cargo-wagon-spritesheet-1.png",
			"__Hyperloop__/graphics/hyperloop-cargo-wagon/cargo-wagon-spritesheet-2.png",
			"__Hyperloop__/graphics/hyperloop-cargo-wagon/cargo-wagon-spritesheet-3.png",
			"__Hyperloop__/graphics/hyperloop-cargo-wagon/cargo-wagon-spritesheet-4.png"
		  },
		  line_length = 4,
		  lines_per_file = 8,
		  shift = {0.7, -0.45}
		},
		rail_category = "hyperloop-rail",
		drive_over_tie_trigger = drive_over_tie(),
		tie_distance = 50,
		working_sound =
		{
		  sound =
		  {
			filename = "__base__/sound/train-wheels.ogg",
			volume = 0.6
		  },
		  match_volume_to_activity = true,
		},
		crash_trigger = crash_trigger(),
		open_sound = { filename = "__base__/sound/machine-open.ogg", volume = 0.85 },
		close_sound = { filename = "__base__/sound/machine-close.ogg", volume = 0.75 },
		sound_minimum_speed = 0.5;
		vehicle_impact_sound =  { filename = "__base__/sound/car-wood-impact.ogg", volume = 1.0 },
	},
}
)